@@ -4,6 +4,7 @@ import os
 import shlex
 import typing
 from pathlib import Path
+from urllib.parse import urljoin
 
 import yaml
 from rhasspyprofile import Profile
@@ -265,6 +266,39 @@ def add_standard_args(
         command.extend(["--username", shlex.quote(str(mqtt_username))])
         command.extend(["--password", shlex.quote(str(mqtt_password))])
 
+    # TLS
+    tls_enabled = profile.get("mqtt.tls.enabled", False)
+    if tls_enabled:
+        command.append("--tls")
+
+        # Certificate Authority certs
+        tls_ca_certs = profile.get("mqtt.tls.ca_certs")
+        if tls_ca_certs:
+            command.extend(["--tls-ca-certs", shlex.quote(str(tls_ca_certs))])
+
+        # CERT_REQUIRED, CERT_OPTIONAL, CERT_NONE
+        tls_cert_reqs = profile.get("mqtt.tls.cert_reqs")
+        if tls_cert_reqs:
+            command.extend(["--tls-cert-reqs", shlex.quote(str(tls_cert_reqs))])
+
+        # PEM
+        tls_certfile = profile.get("mqtt.tls.certfile")
+        if tls_certfile:
+            command.extend(["--tls-certfile", shlex.quote(str(tls_certfile))])
+
+        tls_keyfile = profile.get("mqtt.tls.keyfile")
+        if tls_keyfile:
+            command.extend(["--tls-keyfile", shlex.quote(str(tls_keyfile))])
+
+        # Cipers/version
+        tls_ciphers = profile.get("mqtt.tls.ciphers")
+        if tls_ciphers:
+            command.extend(["--tls-ciphers", shlex.quote(str(tls_ciphers))])
+
+        tls_version = profile.get("mqtt.tls.version")
+        if tls_version:
+            command.extend(["--tls-version", shlex.quote(str(tls_version))])
+
     log_format = profile.get("logging.format", "")
     if log_format:
         command.extend(["--log-format", shlex.quote(str(log_format))])
@@ -811,6 +845,7 @@ def get_speech_to_text(
             _LOGGER.error("speech_to_text.pocketsphinx.acoustic_model is required")
             return []
 
+        # Open transcription
         open_transcription = bool(
             profile.get("speech_to_text.pocketsphinx.open_transcription", False)
         )
@@ -908,6 +943,29 @@ def get_speech_to_text(
                 ]
             )
 
+        # Mixed language model
+        base_lm_fst = profile.get("speech_to_text.pocketsphinx.base_language_model_fst")
+        if base_lm_fst:
+            stt_command.extend(
+                [
+                    "--base-language-model-fst",
+                    shlex.quote(str(write_path(profile, base_lm_fst))),
+                ]
+            )
+
+        base_lm_weight = str(profile.get("speech_to_text.pocketsphinx.mix_weight", ""))
+        if base_lm_weight:
+            stt_command.extend(["--base-language-model-weight", str(base_lm_weight)])
+
+        mix_lm_fst = profile.get("speech_to_text.pocketsphinx.mix_fst")
+        if mix_lm_fst:
+            stt_command.extend(
+                [
+                    "--mixed-language-model-fst",
+                    shlex.quote(str(write_path(profile, mix_lm_fst))),
+                ]
+            )
+
         # Silence detection
         skip_sec = str(profile.get("command.webrtcvad.skip_sec", ""))
         if skip_sec:
@@ -944,6 +1002,7 @@ def get_speech_to_text(
 
         model_dir = write_path(profile, model_dir)
 
+        # Open transcription
         open_transcription = bool(
             profile.get("speech_to_text.kaldi.open_transcription", False)
         )
@@ -1048,6 +1107,29 @@ def get_speech_to_text(
                 ]
             )
 
+        # Mixed language model
+        base_lm_fst = profile.get("speech_to_text.kaldi.base_language_model_fst")
+        if base_lm_fst:
+            stt_command.extend(
+                [
+                    "--base-language-model-fst",
+                    shlex.quote(str(write_path(profile, base_lm_fst))),
+                ]
+            )
+
+        base_lm_weight = str(profile.get("speech_to_text.kaldi.mix_weight", ""))
+        if base_lm_weight:
+            stt_command.extend(["--base-language-model-weight", str(base_lm_weight)])
+
+        mix_lm_fst = profile.get("speech_to_text.kaldi.mix_fst")
+        if mix_lm_fst:
+            stt_command.extend(
+                [
+                    "--mixed-language-model-fst",
+                    shlex.quote(str(write_path(profile, mix_lm_fst))),
+                ]
+            )
+
         # Silence detection
         skip_sec = str(profile.get("command.webrtcvad.skip_sec", ""))
         if skip_sec:
@@ -1177,6 +1259,7 @@ def get_speech_to_text(
             _LOGGER.error("speech_to_text.deepspeech.acoustic_model is required")
             return []
 
+        # Open transcription
         open_transcription = bool(
             profile.get("speech_to_text.deepspeech.open_transcription", False)
         )
@@ -1229,6 +1312,29 @@ def get_speech_to_text(
             # Don't overwrite dictionary or language model during training
             stt_command.append("--no-overwrite-train")
 
+        # Mixed language model
+        base_lm_fst = profile.get("speech_to_text.deepspeech.base_language_model_fst")
+        if base_lm_fst:
+            stt_command.extend(
+                [
+                    "--base-language-model-fst",
+                    shlex.quote(str(write_path(profile, base_lm_fst))),
+                ]
+            )
+
+        base_lm_weight = str(profile.get("speech_to_text.deepspeech.mix_weight", ""))
+        if base_lm_weight:
+            stt_command.extend(["--base-language-model-weight", str(base_lm_weight)])
+
+        mix_lm_fst = profile.get("speech_to_text.deepspeech.mix_fst")
+        if mix_lm_fst:
+            stt_command.extend(
+                [
+                    "--mixed-language-model-fst",
+                    shlex.quote(str(write_path(profile, mix_lm_fst))),
+                ]
+            )
+
         # Silence detection
         skip_sec = str(profile.get("command.webrtcvad.skip_sec", ""))
         if skip_sec:
@@ -1897,26 +2003,13 @@ def get_text_to_speech(
         return tts_command
 
     if tts_system == "picotts":
-        picotts_command = ["pico2wave", "-l", "{lang}"]
-        picotts_command.extend(["-w", "$t", '"$0"'])
-
-        # pico2wave REALLY wants to write to a real file, so we have to wrap it
-        # in a small script to use a temporary one.
-        picotts_command_str = " ".join(str(v) for v in picotts_command)
-        bash_command = [
-            "bash",
-            "-c",
-            shlex.quote(
-                "t=$(mktemp --suffix .wav); function x() {{ rm -f $t; }}; trap x EXIT; "
-                + picotts_command_str
-                + "; cat $t"
-            ),
-        ]
+        picotts_command = ["pico2wave", "-l", "{lang}", "-w", "{file}"]
 
         tts_command = [
             "rhasspy-tts-cli-hermes",
             "--tts-command",
-            shlex.quote(" ".join(str(v) for v in bash_command)),
+            shlex.quote(" ".join(str(v) for v in picotts_command)),
+            "--temporary-wav",
         ]
 
         add_standard_args(
@@ -1929,11 +2022,54 @@ def get_text_to_speech(
             mqtt_password,
         )
 
-        locale = str(profile.get("locale", "")).strip()
+        picotts_language = str(profile.get("text_to_speech.picotts.language", ""))
+        if picotts_language:
+            tts_command.extend(["--language", shlex.quote(str(picotts_language))])
+        else:
+            # Fall back to profile locale
+            locale = str(profile.get("locale", "")).strip()
 
-        if locale:
-            locale = locale.replace("_", "-")
-            tts_command.extend(["--language", shlex.quote(str(locale))])
+            if locale:
+                locale = locale.replace("_", "-")
+                tts_command.extend(["--language", shlex.quote(str(locale))])
+
+        return tts_command
+
+    if tts_system == "nanotts":
+        nanotts_command = ["nanotts", "-v", "{lang}", "-o", "{file}"]
+        tts_command = [
+            "rhasspy-tts-cli-hermes",
+            "--tts-command",
+            shlex.quote(" ".join(str(v) for v in nanotts_command)),
+            "--temporary-wav",
+            "--text-on-stdin",
+        ]
+
+        add_standard_args(
+            profile,
+            tts_command,
+            site_ids,
+            mqtt_host,
+            mqtt_port,
+            mqtt_username,
+            mqtt_password,
+        )
+
+        nanotts_language = str(profile.get("text_to_speech.nanotts.language", ""))
+        if nanotts_language:
+            tts_command.extend(["--language", shlex.quote(str(nanotts_language))])
+        else:
+            # Fall back to profile locale
+            locale = str(profile.get("locale", "")).strip()
+
+            if locale:
+                locale = locale.replace("_", "-")
+                tts_command.extend(["--language", shlex.quote(str(locale))])
+
+        langdir = str(profile.get("text_to_speech.nanotts.langdir", ""))
+
+        if langdir:
+            tts_command.extend(["-l", shlex.quote(os.path.expandvars(str(locale)))])
 
         return tts_command
 
@@ -2016,6 +2152,72 @@ def get_text_to_speech(
 
         return tts_command
 
+    if tts_system == "opentts":
+        url = profile.get("text_to_speech.opentts.url", "").strip()
+        if not url:
+            _LOGGER.error("text_to_speech.opentts.url is required")
+            return []
+
+        voice = profile.get("text_to_speech.opentts.voice", "").strip()
+        if not voice:
+            _LOGGER.error("text_to_speech.opentts.voice is required")
+            return []
+
+        # Oh the things curl can do
+        opentts_command = [
+            "curl",
+            "-sS",
+            "-X",
+            "GET",
+            "-G",
+            "--output",
+            "-",
+            "--data-urlencode",
+            f"voice={voice}",
+            "--data-urlencode",
+            'text="$0"',
+            shlex.quote(urljoin(url, "api/tts")),
+        ]
+
+        # Combine into bash call so we can pass input text as $0
+        bash_command = [
+            "bash",
+            "-c",
+            shlex.quote(" ".join(str(v) for v in opentts_command)),
+        ]
+
+        voices_command = [
+            "curl",
+            "-sS",
+            "-X",
+            "GET",
+            shlex.quote(urljoin(url, "api/voices")),
+            "|",
+            "jq",
+            "--raw-output",
+            shlex.quote('keys[] as $k | "\\($k) \\(.[$k] | .name)"'),
+        ]
+
+        tts_command = [
+            "rhasspy-tts-cli-hermes",
+            "--tts-command",
+            shlex.quote(" ".join(str(v) for v in bash_command)),
+            "--voices-command",
+            shlex.quote(" ".join(str(v) for v in voices_command)),
+        ]
+
+        add_standard_args(
+            profile,
+            tts_command,
+            site_ids,
+            mqtt_host,
+            mqtt_port,
+            mqtt_username,
+            mqtt_password,
+        )
+
+        return tts_command
+
     if tts_system == "command":
         say_program = profile.get("text_to_speech.command.say_program")
         if not say_program:
